import React, { useState, useEffect, useCallback, useRef, useLayoutEffect } from 'react';

interface EditableCellProps {
  value: string;
  onInput: (value: string) => void;
  focusedCell: string | null;
  setFocusedCell: (cellKey: string | null) => void;
  cellKey: string;
  inputType?: 'text' | 'textarea';
  onFocus?: () => void;
  onBlur?: () => void;
  placeholder?: string;
  className?: string;
}

export function EditableCell({
  value,
  onInput,
  focusedCell,
  setFocusedCell,
  cellKey,
  inputType = 'text',
  onFocus,
  onBlur,
  placeholder = '',
  className = ''
}: EditableCellProps) {
  const [localValue, setLocalValue] = useState(value);
  const inputRef = useRef<HTMLInputElement>(null);
  const textareaRef = useRef<HTMLTextAreaElement>(null);
  const lastExternalValueRef = useRef(value);
  const isUserInputRef = useRef(false);
  
  // Update local value when prop changes (only for genuine external updates)
  useEffect(() => {
    // Only update if this is a genuine external change (not from user input)
    if (value !== lastExternalValueRef.current && !isUserInputRef.current) {
      console.log('External value change detected for cell:', cellKey, value);
      setLocalValue(value);
    }
    lastExternalValueRef.current = value;
    
    // Reset user input flag after a short delay
    if (isUserInputRef.current) {
      const timer = setTimeout(() => {
        isUserInputRef.current = false;
      }, 100);
      return () => clearTimeout(timer);
    }
  }, [value, cellKey]);
  
  // Handle focus when this cell becomes focused
  useEffect(() => {
    if (focusedCell === cellKey) {
      const element = inputType === 'textarea' ? textareaRef.current : inputRef.current;
      if (element && document.activeElement !== element) {
        element.focus();
        // Only select text if this is not from user input
        if (!isUserInputRef.current) {
          element.select();
        }
      }
    }
  }, [focusedCell, cellKey, inputType]);

  // Auto-resize textarea based on content
  const autoResize = useCallback(() => {
    if (inputType === 'textarea' && textareaRef.current) {
      const textarea = textareaRef.current;

      // Reset height to auto to get the correct scrollHeight
      textarea.style.height = 'auto';

      // Calculate the required height
      const scrollHeight = textarea.scrollHeight;
      const minHeight = 40; // 2.5rem = 40px
      const maxHeight = 192; // 12rem = 192px

      // Set height within bounds
      const newHeight = Math.min(Math.max(scrollHeight, minHeight), maxHeight);
      textarea.style.height = `${newHeight}px`;

      // Enable scrolling if content exceeds max height
      textarea.style.overflowY = scrollHeight > maxHeight ? 'auto' : 'hidden';
    }
  }, [inputType]);

  // Auto-resize on content change
  useLayoutEffect(() => {
    autoResize();
  }, [localValue, autoResize]);

  // Auto-resize on focus (in case content was updated externally)
  useEffect(() => {
    if (focusedCell === cellKey) {
      autoResize();
    }
  }, [focusedCell, cellKey, autoResize]);
  
  const handleInput = useCallback((e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) => {
    const newValue = e.target.value;
    console.log('Input event for cell:', cellKey, newValue);
    isUserInputRef.current = true; // Mark as user input
    setLocalValue(newValue);
  }, [cellKey]);
  
  const commitInput = useCallback(() => {
    console.log('Committing input for cell:', cellKey, localValue);
    onInput(localValue);
  }, [localValue, onInput, cellKey]);
  
  const handleFocus = useCallback(() => {
    console.log('Focus gained for cell:', cellKey);
    setFocusedCell(cellKey);
    onFocus?.();
  }, [cellKey, setFocusedCell, onFocus]);
  
  const handleBlur = useCallback(() => {
    console.log('Focus lost for cell:', cellKey);
    setFocusedCell(null);
    commitInput();
    onBlur?.();
  }, [setFocusedCell, commitInput, onBlur, cellKey]);
  
  const handleKeyDown = useCallback((e: React.KeyboardEvent) => {
    if (e.key === 'Enter' && inputType === 'text') {
      // For text inputs, commit on Enter
      e.preventDefault();
      commitInput();
      setFocusedCell(null);
    } else if (e.key === 'Escape') {
      // Reset to original value on Escape
      setLocalValue(value);
      setFocusedCell(null);
    }
  }, [inputType, commitInput, setFocusedCell, value]);
  
  const baseClassName = `
    w-full h-full bg-transparent text-inherit
    border-none outline-none focus:outline-none resize-none
    placeholder:text-gray-400 dark:placeholder:text-gray-500
    leading-relaxed
    ${className}
  `.trim();
  
  if (inputType === 'textarea') {
    return (
      <div className="editable-cell">
        <textarea
          ref={textareaRef}
          value={localValue}
          onChange={handleInput}
          onFocus={handleFocus}
          onBlur={handleBlur}
          onKeyDown={handleKeyDown}
          onInput={autoResize}
          placeholder={placeholder}
          className={baseClassName}
          style={{ 
            minHeight: '2.5rem',
            resize: 'none',
            overflow: 'hidden'
          }}
          rows={1} // Start with single row
        />
      </div>
    );
  }

  return (
    <div className="editable-cell">
      <input
        ref={inputRef}
        type="text"
        value={localValue}
        onChange={handleInput}
        onFocus={handleFocus}
        onBlur={handleBlur}
        onKeyDown={handleKeyDown}
        placeholder={placeholder}
        className={baseClassName}
      />
    </div>
  );
}