import React, { useState, useEffect, useCallback, useRef } from 'react';
import { createPortal } from 'react-dom';
import { WikidataSuggestion } from '@/types/database';

interface TypeaheadInputProps {
  value: string;
  onInput: (value: string) => void;
  onSelect: (suggestion: WikidataSuggestion) => void;
  fetchSuggestions: (query: string) => Promise<WikidataSuggestion[]>;
  onFocus?: () => void;
  onBlur?: () => void;
  placeholder?: string;
  className?: string;
}

interface DropdownPosition {
  top: number;
  left: number;
  width: number;
}

export function TypeaheadInput({
  value,
  onInput,
  onSelect,
  fetchSuggestions,
  onFocus,
  onBlur,
  placeholder = '',
  className = ''
}: TypeaheadInputProps) {
  const [localValue, setLocalValue] = useState(value);
  const [suggestions, setSuggestions] = useState<WikidataSuggestion[]>([]);
  const [showSuggestions, setShowSuggestions] = useState(false);
  const [selectedIndex, setSelectedIndex] = useState(-1);
  const [isLoading, setIsLoading] = useState(false);
  const [isExternalUpdate, setIsExternalUpdate] = useState(false);
  const [dropdownPosition, setDropdownPosition] = useState<DropdownPosition>({ top: 0, left: 0, width: 0 });
  const [isDarkMode, setIsDarkMode] = useState(false);
  
  const inputRef = useRef<HTMLInputElement>(null);
  const suggestionsRef = useRef<HTMLDivElement>(null);
  const timeoutRef = useRef<NodeJS.Timeout | undefined>(undefined);
  
  // Detect dark mode
  useEffect(() => {
    const checkDarkMode = () => {
      // Check if dark class is on html or body element
      const isDark = document.documentElement.classList.contains('dark') || 
                    document.body.classList.contains('dark') ||
                    // Fallback: check system preference
                    window.matchMedia('(prefers-color-scheme: dark)').matches;
      setIsDarkMode(isDark);
    };
    
    // Initial check
    checkDarkMode();
    
    // Watch for changes to the dark class
    const observer = new MutationObserver(checkDarkMode);
    observer.observe(document.documentElement, {
      attributes: true,
      attributeFilter: ['class']
    });
    observer.observe(document.body, {
      attributes: true,
      attributeFilter: ['class']
    });
    
    // Watch for system theme changes
    const mediaQuery = window.matchMedia('(prefers-color-scheme: dark)');
    mediaQuery.addEventListener('change', checkDarkMode);
    
    return () => {
      observer.disconnect();
      mediaQuery.removeEventListener('change', checkDarkMode);
    };
  }, []);
  
  // Calculate dropdown position relative to viewport
  const calculateDropdownPosition = useCallback((): DropdownPosition => {
    if (!inputRef.current) {
      return { top: 0, left: 0, width: 0 };
    }
    
    const inputRect = inputRef.current.getBoundingClientRect();
    const viewportHeight = window.innerHeight;
    const dropdownHeight = 384; // max-h-96 = 24rem = 384px
    
    // Calculate if dropdown should appear above or below input
    const spaceBelow = viewportHeight - inputRect.bottom;
    const spaceAbove = inputRect.top;
    const shouldShowAbove = spaceBelow < dropdownHeight && spaceAbove > spaceBelow;
    
    return {
      top: shouldShowAbove 
        ? inputRect.top - Math.min(dropdownHeight, spaceAbove) + window.scrollY
        : inputRect.bottom + window.scrollY,
      left: inputRect.left + window.scrollX,
      width: inputRect.width
    };
  }, []);
  
  // Update dropdown position when showing suggestions
  const updateDropdownPosition = useCallback(() => {
    if (showSuggestions) {
      const position = calculateDropdownPosition();
      setDropdownPosition(position);
    }
  }, [showSuggestions, calculateDropdownPosition]);
  
  // Update local value when prop changes (including external updates)
  useEffect(() => {
    if (value !== localValue) {
      setLocalValue(value);
      setIsExternalUpdate(true);
      // Clear suggestions when externally updated
      setSuggestions([]);
      setShowSuggestions(false);
      setSelectedIndex(-1);
    }
  }, [value, localValue]);

  // Reset external update flag after a short delay
  useEffect(() => {
    if (isExternalUpdate) {
      const timer = setTimeout(() => {
        setIsExternalUpdate(false);
      }, 100);
      return () => clearTimeout(timer);
    }
  }, [isExternalUpdate]);
  
  // Update dropdown position on scroll and resize
  useEffect(() => {
    if (showSuggestions) {
      const handleScroll = () => updateDropdownPosition();
      const handleResize = () => updateDropdownPosition();
      
      window.addEventListener('scroll', handleScroll, true);
      window.addEventListener('resize', handleResize);
      
      return () => {
        window.removeEventListener('scroll', handleScroll, true);
        window.removeEventListener('resize', handleResize);
      };
    }
  }, [showSuggestions, updateDropdownPosition]);
  
  // Debounced suggestion fetching
  const debouncedFetchSuggestions = useCallback(async (query: string) => {
    if (timeoutRef.current) {
      clearTimeout(timeoutRef.current);
    }
    
    timeoutRef.current = setTimeout(async () => {
      if (query.trim().length < 2) {
        setSuggestions([]);
        setShowSuggestions(false);
        setIsLoading(false);
        return;
      }
      
      setIsLoading(true);
      try {
        console.log('Fetching suggestions for:', query);
        const results = await fetchSuggestions(query);
        setSuggestions(results);
        if (results.length > 0) {
          setShowSuggestions(true);
          // Calculate position when showing suggestions
          const position = calculateDropdownPosition();
          setDropdownPosition(position);
        }
        setSelectedIndex(-1);
      } catch (error) {
        console.error('Error fetching suggestions:', error);
        setSuggestions([]);
        setShowSuggestions(false);
      } finally {
        setIsLoading(false);
      }
    }, 300); // 300ms debounce
  }, [fetchSuggestions, calculateDropdownPosition]);
  
  // Handle input changes
  const handleInput = useCallback((e: React.ChangeEvent<HTMLInputElement>) => {
    const newValue = e.target.value;
    setLocalValue(newValue);
    onInput(newValue);
    
    // Only fetch suggestions if this is user input, not external update
    if (!isExternalUpdate) {
      debouncedFetchSuggestions(newValue);
    }
  }, [onInput, debouncedFetchSuggestions, isExternalUpdate]);
  
  // Handle keyboard navigation
  const handleKeyDown = useCallback((e: React.KeyboardEvent<HTMLInputElement>) => {
    if (!showSuggestions || suggestions.length === 0) {
      if (e.key === 'Enter') {
        e.preventDefault();
        // Commit the current value
        onInput(localValue);
      }
      return;
    }
    
    switch (e.key) {
      case 'ArrowDown':
        e.preventDefault();
        setSelectedIndex(prev => 
          prev < suggestions.length - 1 ? prev + 1 : 0
        );
        break;
        
      case 'ArrowUp':
        e.preventDefault();
        setSelectedIndex(prev => 
          prev > 0 ? prev - 1 : suggestions.length - 1
        );
        break;
        
      case 'Enter':
        e.preventDefault();
        if (selectedIndex >= 0 && selectedIndex < suggestions.length) {
          const selectedSuggestion = suggestions[selectedIndex];
          handleSelect(selectedSuggestion);
        } else {
          // No suggestion selected, just commit the current value
          onInput(localValue);
          setShowSuggestions(false);
        }
        break;
        
      case 'Escape':
        e.preventDefault();
        setShowSuggestions(false);
        setSelectedIndex(-1);
        inputRef.current?.blur();
        break;
        
      case 'Tab':
        // Allow tab to close suggestions
        setShowSuggestions(false);
        setSelectedIndex(-1);
        break;
    }
  }, [showSuggestions, suggestions, selectedIndex, localValue, onInput]);
  
  // Handle suggestion selection
  const handleSelect = useCallback((suggestion: WikidataSuggestion) => {
    console.log('Suggestion selected:', suggestion);

    // Update local value with the selected suggestion
    const newValue = suggestion.label || '';
    setLocalValue(newValue);

    // Call the selection handler (this will update the parent with all Wikidata info)
    // This handles name, description, and wikidataId updates
    onSelect(suggestion);

    // Hide suggestions
    setShowSuggestions(false);
    setSelectedIndex(-1);

    // Keep focus on input
    inputRef.current?.focus();
  }, [onSelect]);
  
  // Handle focus
  const handleFocus = useCallback(() => {
    console.log('TypeaheadInput focused');
    onFocus?.();
    
    // Show suggestions if we have them and there's a value
    if (suggestions.length > 0 && localValue.trim().length >= 2) {
      setShowSuggestions(true);
      const position = calculateDropdownPosition();
      setDropdownPosition(position);
    }
  }, [onFocus, suggestions.length, localValue, calculateDropdownPosition]);
  
  // Handle blur
  const handleBlur = useCallback((e: React.FocusEvent<HTMLInputElement>) => {
    // Delay hiding suggestions to allow for clicks on suggestions
    setTimeout(() => {
      // Check if the new focus target is within our suggestions
      const relatedTarget = e.relatedTarget as HTMLElement;
      if (!suggestionsRef.current?.contains(relatedTarget)) {
        setShowSuggestions(false);
        setSelectedIndex(-1);
        onBlur?.();
      }
    }, 150);
  }, [onBlur]);
  
  // Handle clicking on suggestions
  const handleSuggestionClick = useCallback((suggestion: WikidataSuggestion) => {
    handleSelect(suggestion);
  }, [handleSelect]);
  
  // Handle clicking outside to close suggestions
  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      const target = event.target as HTMLElement;
      if (
        inputRef.current && 
        !inputRef.current.contains(target) &&
        suggestionsRef.current &&
        !suggestionsRef.current.contains(target)
      ) {
        setShowSuggestions(false);
        setSelectedIndex(-1);
      }
    };
    
    if (showSuggestions) {
      document.addEventListener('mousedown', handleClickOutside);
      return () => document.removeEventListener('mousedown', handleClickOutside);
    }
  }, [showSuggestions]);
  
  // Cleanup timeout on unmount
  useEffect(() => {
    return () => {
      if (timeoutRef.current) {
        clearTimeout(timeoutRef.current);
      }
    };
  }, []);
  
  const baseInputClassName = `
    w-full h-full bg-transparent text-inherit
    border-none outline-none focus:outline-none
    placeholder:text-gray-400 dark:placeholder:text-gray-500
    ${className}
  `.trim();
  
  // Portal-based dropdown component
  const DropdownPortal = () => {
    if (!showSuggestions || suggestions.length === 0) return null;
    
    // Theme-aware colors
    const dropdownBgColor = isDarkMode ? 'rgba(31, 41, 55, 1)' : 'rgba(255, 255, 255, 1)'; // gray-800 : white
    const borderColor = isDarkMode ? 'rgba(75, 85, 99, 1)' : 'rgba(209, 213, 219, 1)'; // gray-600 : gray-300
    
    return createPortal(
      <div
        ref={suggestionsRef}
        className="fixed border-2 rounded-xl shadow-2xl max-h-96 overflow-y-auto"
        style={{
          top: dropdownPosition.top + 8, // 8px gap below input
          left: dropdownPosition.left,
          width: dropdownPosition.width,
          zIndex: 9999, // Higher than table headers (z-index: 10)
          backgroundColor: dropdownBgColor,
          borderColor: borderColor,
          boxShadow: isDarkMode 
            ? '0 20px 25px -5px rgba(0, 0, 0, 0.4), 0 10px 10px -5px rgba(0, 0, 0, 0.2), 0 0 0 1px rgba(0, 0, 0, 0.3)'
            : '0 20px 25px -5px rgba(0, 0, 0, 0.1), 0 10px 10px -5px rgba(0, 0, 0, 0.04), 0 0 0 1px rgba(0, 0, 0, 0.05)'
        }}
      >
        <div className="py-2">
          {suggestions.map((suggestion, index) => {
            const isSelected = index === selectedIndex;
            const label = suggestion.label || '';
            const description = suggestion.description || '';
            
            // Theme-aware item colors
            const itemBgColor = isSelected 
              ? (isDarkMode ? 'rgba(30, 58, 138, 0.3)' : 'rgba(239, 246, 255, 1)') // blue-900/30 : blue-50
              : dropdownBgColor;
            
            return (
              <div
                key={suggestion.id}
                className={`
                  relative px-4 py-4 cursor-pointer transition-all duration-200 ease-in-out
                  border-l-4 mx-2 my-1 rounded-lg
                  ${isSelected 
                    ? 'border-l-blue-500 shadow-md' 
                    : 'border-l-transparent hover:shadow-sm'
                  }
                  ${isSelected 
                    ? '' 
                    : (isDarkMode ? 'hover:bg-gray-700' : 'hover:bg-gray-50')
                  }
                  ${!isSelected && isDarkMode ? 'hover:border-l-gray-500' : ''}
                  ${!isSelected && !isDarkMode ? 'hover:border-l-gray-300' : ''}
                `}
                onClick={() => handleSuggestionClick(suggestion)}
                onMouseEnter={() => setSelectedIndex(index)}
                style={{
                  backgroundColor: itemBgColor
                }}
              >
                <div className="flex items-start justify-between">
                  <div className="flex-1 min-w-0 space-y-2">
                    {/* Main label */}
                    <div className={`font-semibold text-base leading-tight ${
                      isSelected 
                        ? (isDarkMode ? 'text-blue-100' : 'text-blue-900')
                        : (isDarkMode ? 'text-gray-100' : 'text-gray-900')
                    }`}>
                      {label}
                    </div>
                    
                    {/* Description */}
                    {description && (
                      <div className={`text-sm leading-relaxed ${
                        isSelected 
                          ? (isDarkMode ? 'text-blue-200' : 'text-blue-700')
                          : (isDarkMode ? 'text-gray-300' : 'text-gray-600')
                      }`}>
                        {description}
                      </div>
                    )}
                    
                    {/* Wikidata ID */}
                    <div className="flex items-center space-x-2">
                      <span className={`inline-flex items-center px-2 py-1 rounded-full text-xs font-medium border ${
                        isSelected 
                          ? (isDarkMode 
                              ? 'bg-blue-800 text-blue-100 border-blue-600' 
                              : 'bg-blue-100 text-blue-800 border-blue-200'
                            )
                          : (isDarkMode 
                              ? 'bg-gray-700 text-gray-300 border-gray-600' 
                              : 'bg-gray-100 text-gray-700 border-gray-200'
                            )
                      }`}>
                        <svg className="w-3 h-3 mr-1" fill="currentColor" viewBox="0 0 20 20">
                          <path fillRule="evenodd" d="M4 4a2 2 0 00-2 2v8a2 2 0 002 2h12a2 2 0 002-2V6a2 2 0 00-2-2H4zm2 6a2 2 0 104 0 2 2 0 00-4 0zm8-2a2 2 0 11-4 0 2 2 0 014 0z" clipRule="evenodd" />
                        </svg>
                        {suggestion.id}
                      </span>
                    </div>
                  </div>
                  
                  {/* Selection indicator */}
                  <div className="ml-4 flex-shrink-0">
                    <div className={`w-6 h-6 rounded-full flex items-center justify-center transition-all duration-200 ${
                      isSelected 
                        ? (isDarkMode ? 'bg-blue-600 text-white shadow-lg' : 'bg-blue-500 text-white shadow-lg')
                        : (isDarkMode ? 'bg-gray-600 text-gray-300' : 'bg-gray-200 text-gray-400')
                    }`}>
                      <svg className="w-3 h-3" fill="none" stroke="currentColor" viewBox="0 0 24 24">
                        <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M9 5l7 7-7 7" />
                      </svg>
                    </div>
                  </div>
                </div>
                
                {/* Subtle bottom border for separation */}
                {index < suggestions.length - 1 && (
                  <div className={`absolute bottom-0 left-4 right-4 h-px ${
                    isDarkMode ? 'bg-gray-600' : 'bg-gray-200'
                  }`}></div>
                )}
              </div>
            );
          })}
        </div>
      </div>,
      document.body
    );
  };
  
  return (
    <div className="relative w-full h-full flex items-center">
      <input
        ref={inputRef}
        type="text"
        value={localValue}
        onChange={handleInput}
        onKeyDown={handleKeyDown}
        onFocus={handleFocus}
        onBlur={handleBlur}
        placeholder={placeholder}
        className={baseInputClassName}
        autoComplete="off"
      />
      
      {/* Loading indicator */}
      {isLoading && (
        <div className="absolute right-3 top-1/2 transform -translate-y-1/2 z-10">
          <div className="animate-spin rounded-full h-4 w-4 border-2 border-blue-500 border-t-transparent"></div>
        </div>
      )}
      
      {/* Portal-based suggestions dropdown */}
      <DropdownPortal />
    </div>
  );
}